<?php
/**
 * @license http://creativecommons.org/licenses/by-sa/3.0/ Creative Commons Attribution-Share Alike 3.0 Unported
 * @author John Noel <john.noel@chaostangent.com>
 * @link http://chaostangent.com
 * @package Life
 */

/**
 * John Horton Conway's Game of Life in PHP
 * @author John Noel <john.noel@chaostangent.com>
 * @version 1.1
 */
class Life
{
	private $width, $height;

	/**
	 * Construct the Game of Life
	 * @param int $width The width of the game world
	 * @param int $height The height of the game world
	 */
	public function __construct($width, $height)
	{
		$this->width = $width; $this->height = $height;
	}

	/**
	 * Start the Game of Life
	 * @param string $seed A string representing the seed
	 * @param int $generations The number of generations to calculate
	 * @param string $outputDirectory Where to put the GIF files
	 * @param array $foreground A triple of integers representing the RGB values for a foreground colour
	 * @param array $background As $foreground except for the background colour
	 */
	public function start($seed, $generations, $outputDirectory, array $foreground, array $background)
	{
		// pre-fill with dead cells
		$currentState = array_fill(0, $this->height, array_fill(0, $this->width, 0));

		$this->seed($seed, $currentState);
		//$this->outputState($currentState, "Seed");
		$this->outputStateToGif($currentState, 0, $outputDirectory, $foreground, $background);

		for($i = 1; $i <= $generations; $i++)
		{
			$newState = $this->tick($currentState);
			//$this->outputState($newState, $i);
			$this->outputStateToGif($newState, $i, $outputDirectory, $foreground, $background);

			if($this->isInactive($newState) || $this->isEqual($currentState, $newState))
			{
				break;
			}

			$currentState = $newState;
		}
	}

	/**
	 * Populate a state with a given seed
	 * @param string $seed A string of 0, 1 or \n representing the desired seed
	 * @param array $currentState A reference to the current state to populate
	 */
	private function seed($seed, array &$currentState)
	{
		$rows = explode("\n", $seed);
		$height = count($rows);
		$width = 0;
		foreach($rows AS $row)
		{
			$s = strlen(trim($row));
			$width = ($s > $width) ? $s : $width;
		}

		// centre the seed
		$x = round($this->width / 2) - round($width / 2);
		$y = round($this->height / 2) - round($height / 2);

		for($h = 0; $h < $height; $h++)
		{
			for($w = 0; $w < $width; $w++)
			{
				$currentState[$x + $w][$y + $h] = substr(trim($rows[$h]), $w, 1);
			}
		}
	}

	/**
	 * Advances the Game of Life one generation
	 * @param array $currentState The state to advance one generation
	 * @return array The next state
	 */
	private function tick(array $currentState)
	{
		set_time_limit(30);

		// copy
		$newState = $currentState;
		$width = count($currentState[0]); $height = count($currentState);

		for($h = 0; $h < $height; $h++)
		{
			for($w = 0; $w < $width; $w++)
			{
				$neighbours = 0;
				for($i = 0; $i < 8; $i++)
				{
					$newH = $h; $newW = $w;
					switch($i)
					{
						case 0:
							$newW -= 1; $newH -= 1;
							break;
						case 1:
							$newH -= 1;
							break;
						case 2:
							$newW += 1; $newH -=1;
							break;
						case 3:
							$newW += 1;
							break;
						case 4:
							$newW += 1; $newH += 1;
							break;
						case 5:
							$newH += 1;
							break;
						case 6:
							$newW -= 1; $newH += 1;
							break;
						case 7:
							$newW -= 1;
							break;
					}

					$newW = ($newW < 0) ? ($width + $newW) : $newW;
					$newW = ($newW >= $width) ? ($newW - $width) : $newW;

					$newH = ($newH < 0) ? ($height + $newH) : $newH;
					$newH = ($newH >= $height) ? ($newH - $height) : $newH;

					$neighbours += ($currentState[$newW][$newH]);
				}

				if($currentState[$w][$h] == 1)
				{
					if(($neighbours < 2) || ($neighbours > 3))
					{
						$newState[$w][$h] = 0;
					}
				}
				else
				{
					if($neighbours == 3)
					{
						$newState[$w][$h] = 1;
					}
				}
			}
		}

		return $newState;
	}

	/**
	 * Given a state this will directly output an HTML table of the state
	 * Inadvisable to use this for very large grids or many states at the same time
	 * @param array $state The state to output
	 * @param string|int $generation The current generation, used in a table caption
	 */
	private function outputState(array $state, $generation)
	{
		$width = count($state[0]); $height = count($state);

		echo "<table cellspacing=\"0\" cellpadding=\"0\"><caption>Generation {$generation}</caption><thead><tr><th>&nbsp;</td>";
		for($w = 0; $w < $width; $w++)
		{
			echo "<th>{$w}</th>";
		}

		for($h = 0; $h < $height; $h++)
		{
			echo "<tr><th>{$h}</th>";
			for($w = 0; $w < $width; $w++)
			{
				echo "<td".(($state[$w][$h]) ? " class=\"on\"" : "").">&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</td>";
			}
			echo "</tr>";
		}
		echo "</table>\n";
	}

	/**
	 * Given a state this will output a GIF file using the given $foreground and $background colours
	 * @param array $state The state to output
	 * @param int $generation The current generation, used for the filename
	 * @param string $outputDir Where to output the GIF file (must be writable)
	 * @param array $foreground A triple of integers representing the RGB values for a foreground colour
	 * @param array $background As $foreground except for the background colour
	 */
	private function outputStateToGif(array $state, $generation, $outputDir, $foreground, $background)
	{
		$width = count($state[0]); $height = count($state);

		$img = imagecreate($width * 8, $height * 8);
		imagefill($img, 0, 0, imagecolorallocate($img, $background[0], $background[1], $background[2]));

		$onColour = imagecolorallocate($img, $foreground[0], $foreground[1], $foreground[2]);

		for($h = 0; $h < $height; $h++)
		{
			for($w = 0; $w < $width; $w++)
			{
				if($state[$w][$h] == 1)
				{
					imagefilledrectangle($img, $w*8, $h*8, $w*8+6, $h*8+6, $onColour);
				}
			}
		}

		$outputDir = rtrim($outputDir, "/");
		$filename = sprintf("%04d", $generation);

		imagegif($img, "{$outputDir}/{$filename}.gif");
		imagedestroy($img);
	}

	/**
	 * Checks whether a given state is entirely inactive - all dead cells
	 * @param array $state The current 2D array state
	 * @return bool Whether this state is inactive or not
	 * @since 1.1
	 */
	private function isInactive(array $state)
	{
		foreach($state AS $row)
		{
			foreach($row AS $cell)
			{
				if($cell == 1)
				{
					return false;
				}
			}
		}

		return true;
	}

	/**
	 * Checks whether two states are equivalent, kind of like a recursive array_diff
	 * @param array $state1
	 * @param array $state2
	 * @return bool Whether the two states are equal
	 * @since 1.1
	 */
	private function isEqual(array $state1, array $state2)
	{
		foreach($state1 AS $h => $row)
		{
			if($row !== $state2[$h])
			{
				return false;
			}
		}

		return true;
	}
}