<?php
/**
 * @license http://creativecommons.org/licenses/by-sa/3.0/ Creative Commons Attribution-Share Alike 3.0 Unported
 * @author John Noel <john.noel@irisassociates.com>
 * @link http://chaostangent.com
 * @package Life
 */

require_once("inc/utf8.inc");

/**
 * Various functions for helping to seed and customise the game of life
 * @author John Noe <john.noel@irisassociates.com>
 * @version 1.0
 */
class Seed
{
	/**
	 * Generate a seed of a specified width and height given an e-mail address
	 * @param string $email The e-mail address to generate from
	 * @param int $width The desired width of the seed
	 * @param int $height The desired height of the seed
	 * @return array A 2D array representing the seed
	 */
	public static function seedFromEmail($email, $width, $height)
	{
		$seed = array();
		$pieces = utf8ToUnicode($email);

		$previous = 124;
		foreach($pieces AS $piece)
		{
			$k = intval(sprintf("%01b", (($piece * $previous) >> PHP_INT_SIZE) & 0x1));
			$seed[] = (count($seed) < ($width * $height)) ? $k : (array_shift($seed) & $k);
			$previous = $piece;
		}

		if(count($seed) < ($width * $height))
		{
			$seed = array_merge($seed, array_fill(0, ($width * $height) - count($seed), 0));
		}

		$seedArray = array();
		for($i = 0; $i < $height; $i++)
		{
			$seedArray[$i] = array_slice($seed, $i * $width, $width);
		}

		return $seedArray;
	}

	/**
	 * Generate a set of foreground and background colours from an e-mail address
	 * @param string $email The e-mail address to generate from
	 * @return array An array containing two triples of integers representing the foreground ([0]) and background ([1]) colours
	 */
	public static function coloursFromEmail($email)
	{
		list($local, $fqdn) = explode("@", $email);
		$multiplier = array_sum(utf8ToUnicode($email));

		$localProduct = array_sum(utf8ToUnicode($local)) * $multiplier;
		$fqdnProduct = array_sum(utf8ToUnicode($fqdn)) * $multiplier;

		$fg = array(($localProduct >> 16) & 0xFF, ($localProduct >> 8) & 0xFF, $localProduct & 0xFF);
		$bg = array(($fqdnProduct >> 16) & 0xFF, ($fqdnProduct >> 8) & 0xFF, $fqdnProduct & 0xFF);

		return array($fg, $bg);
	}

	/**
	 * Generate a single colour from an e-mail address
	 * @param string $email The e-mail address to generate from
	 * @return array A triple of integers representing the RGB components of the colour
	 */
	public static function colourFromEmail($email)
	{
		list($local, $fqdn) = explode("@", $email);

		$red = array_sum(utf8ToUnicode($local)) & 0xFF;
		$green = array_sum(utf8ToUnicode($email)) & 0xFF;
		$blue = array_sum(utf8ToUnicode($fqdn)) & 0xFF;

		return array($red, $green, $blue);
	}

	/**
	 * Outputs a seed using an HTML table taking into account the foreground and background colours
	 * @param array $seed The seed to output
	 * @param string $string How the seed was generated or simple just a caption
	 * @param array $fg A triple of integers representing the RGB values for a foreground colour
	 * @param array $bg As $fg except for the background colour
	 */
	public static function outputSeed(array $seed, $string, array $fg, array $bg)
	{
		$width = count($seed[0]);
		$height = count($seed);

		echo "<table cellspacing=\"0\" cellpadding=\"0\" style=\"background: rgb({$bg[0]},{$bg[1]},{$bg[2]});\"><caption>{$string}</caption><thead><tr><th>&nbsp;</td>";
		for($w = 0; $w < $width; $w++)
		{
			echo "<th>{$w}</th>";
		}

		for($h = 0; $h < $height; $h++)
		{
			echo "<tr><th>{$h}</th>";
			for($w = 0; $w < $width; $w++)
			{
				echo "<td".(($seed[$w][$h]) ? " style=\"background: rgb({$fg[0]},{$fg[1]},{$fg[2]});\"" : "").">&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</td>";
			}
			echo "</tr>";
		}
		echo "</table>\n";
	}
}